//
//  ApiKeyTableViewController.swift
//  SampleApp
//
//  Created by Federico Zanetello on 6/27/19.
//  Copyright © 2019 David Zhang. All rights reserved.
//

import UIKit

final class ApiKeyTableViewController: UITableViewController, DataSourceListener {
    private lazy var dataSource = ApiKeysDataSource()

    private enum Section: CaseIterable {
        case apiKeys
        case add
    }

    // MARK: UITableViewDataSource

    override func viewDidLoad() {
        super.viewDidLoad()
        dataSource.dataSourceObserver = self
    }

    override func numberOfSections(in tableView: UITableView) -> Int {
        return Section.allCases.count
    }

    override func tableView(_ tableView: UITableView,
                            numberOfRowsInSection section: Int) -> Int {
        switch Section.allCases[section] {
        case .apiKeys:
            return dataSource.numberOfKeys()
        case .add:
            return 1
        }
    }

    override func tableView(_ tableView: UITableView,
                            cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        switch Section.allCases[indexPath.section] {
        case .apiKeys:
            return self.tableView(tableView, apiCellForRowAt: indexPath)
        case .add:
            return self.tableView(tableView, addApiCellForRowAt: indexPath)
        }
    }

    private func tableView(_ tableView: UITableView,
                           apiCellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let apiCell: UITableViewCell = tableView.dequeueReusableCell(withIdentifier: "basicCell",
                                                                     for: indexPath)
        apiCell.textLabel?.text = dataSource.key(for: indexPath)
        return apiCell
    }

    private func tableView(_ tableView: UITableView,
                           addApiCellForRowAt indexPath: IndexPath) -> UITableViewCell {
        return tableView.dequeueReusableCell(withIdentifier: "buttonCell",
                                             for: indexPath)
    }

    override func tableView(_ tableView: UITableView,
                            titleForHeaderInSection section: Int) -> String? {
        switch Section.allCases[section] {
        case .apiKeys:
            return "Api Keys"
        case .add:
            return nil
        }
    }

    override func tableView(_ tableView: UITableView,
                            titleForFooterInSection section: Int) -> String? {
        switch Section.allCases[section] {
        case .apiKeys:
            return "Tap on any api key to launch the sample app. Swipe left to remove an api key."
        case .add:
            return nil
        }
    }

    // MARK: UITableViewDelegate

    override func tableView(_ tableView: UITableView,
                            didSelectRowAt indexPath: IndexPath) {
        defer { tableView.deselectRow(at: indexPath, animated: true) }

        switch Section.allCases[indexPath.section] {
        case .apiKeys:
            if let apiKey = dataSource.key(for: indexPath) {
                UserDefaults.standard.currentApiKey = apiKey
                presentChannelSelectionViewController()
            } else {
                assertionFailure("tapped on a impossible index")
            }
        case .add:
            addNewApiKey()
        }
    }

    private func addNewApiKey() {
        let alertController = UIAlertController(title: "Add New Api Key",
                                                message: "",
                                                preferredStyle: .alert)

        alertController.addTextField { textField in
            textField.placeholder = "Type the api key here"
        }

        let addAction = UIAlertAction(title: "Add",
                                      style: .default) { [weak self] _ in
                                        guard
                                            let apiKey = alertController
                                                .textFields?
                                                .first?
                                                .text?
                                                .trimmingCharacters(in: .whitespacesAndNewlines),
                                            !apiKey.isEmpty else { return }

                                        self?.dataSource.addKey(apiKey)
        }

        alertController.addAction(addAction)
        let cancelAction = UIAlertAction(title: "Cancel", style: .cancel, handler: nil)
        alertController.addAction(cancelAction)
        present(alertController, animated: true, completion: nil)
    }

    private func presentChannelSelectionViewController() {
        let dashboardController = MainTabViewController()
        dashboardController.modalPresentationStyle = .fullScreen
        
        present(dashboardController, animated: true, completion: nil)
    }

    override func tableView(_ tableView: UITableView,
                            canEditRowAt indexPath: IndexPath) -> Bool {
        let section: Section = Section.allCases[indexPath.section]
        return section == .apiKeys
    }

    override func tableView(_ tableView: UITableView,
                            commit editingStyle: UITableViewCell.EditingStyle,
                            forRowAt indexPath: IndexPath) {
        guard editingStyle == .delete else { return }

        dataSource.removeKey(at: indexPath.row)
    }

    // MARK: DataSourceListener

    func dataSourceUpdated() {
        tableView.reloadData()
    }
}
