//
//  ChannelListDataSource.swift
//  SampleApp
//
//  Created by Federico Zanetello on 4/26/19.
//  Copyright © 2019 David Zhang. All rights reserved.
//

import EkoChat

final class ChannelListDataSource {
    private let repository: EkoChannelRepository

    private var channelsCollection: EkoCollection<EkoChannel>?
    private var channelsToken: EkoNotificationToken? {
        didSet {
            oldValue?.invalidate()
        }
    }

    weak var dataSourceObserver: DataSourceListener?

    init(channelRepository: EkoChannelRepository) {
        self.repository = channelRepository
    }

    func newQuery(channelType: EkoChannelType, channelFilter: EkoChannelQueryFilter, includingTags: [String], excludingTags: [String], channelTypeName: Set<String> = Set(arrayLiteral: "standard", "private")) {
        switch channelType {
        case .private:
            let builder = EkoPrivateChannelQueryBuilder(includingTags: includingTags, excludingTags: excludingTags)
            channelsCollection = repository.channelCollection().privateType(with: builder).query()
        case .standard:
            let builder = EkoStandardChannelQueryBuilder(channelQueryFilter: channelFilter, includingTags: includingTags, excludingTags: excludingTags)
            channelsCollection = repository.channelCollection().standardType(with: builder).query()
        case .byTypes:
            let builderSet: Set<String> = channelTypeName
            let builder = EkoByTypesChannelQueryBuilder(types: builderSet, channelQueryFilter: channelFilter, includingTags: includingTags, excludingTags: excludingTags)
            channelsCollection = repository.channelCollection().byTypes(with: builder).query()
        case .broadcast:
            let builder = EkoBroadcastChannelQueryBuilder(channelQueryFilter: channelFilter, includingTags: includingTags, excludingTags: excludingTags)
            channelsCollection = repository.channelCollection().broadcast(with: builder).query()
        case .conversation:
            let builder = EkoConversationChannelQueryBuilder(includingTags: nil, excludingTags: nil)
            channelsCollection = repository.channelCollection().conversation(with: builder).query()
        @unknown default:
            fatalError()
        }
        
        dataSourceObserver?.dataSourceUpdated()
        channelsToken = channelsCollection?.observe { [weak self] _, _, _ in
            self?.dataSourceObserver?.dataSourceUpdated()
        }
    }

    func numberOfChannels() -> Int {
        return Int(channelsCollection?.count() ?? 0)
    }

    func channel(for indexPath: IndexPath) -> EkoChannel? {
        if
            let channelsCollection = self.channelsCollection,
            channelsCollection.count() > indexPath.row {
            return channelsCollection.object(at: UInt(indexPath.row))
        } else {
            channelsCollection?.nextPage()
            return nil
        }
    }

    func loadMore() {
        channelsCollection?.nextPage()
    }
}
