//
//  ChannelListTableViewController.swift
//  SampleApp
//
//  Created by Federico Zanetello on 10/4/18.
//  Copyright © 2018 David Zhang. All rights reserved.
//

import UIKit

private let cellIdentifier = "SampleAppTableViewCell"

final class ChannelListTableViewController: UITableViewController, DataSourceListener {
    // To be injected.
    var repository: EkoChannelRepository! {
        didSet {
            dataSource = ChannelListDataSource(channelRepository: repository)
            dataSource?.dataSourceObserver = self
        }
    }

    weak var delegate: ChannelsTableViewControllerDelegate?
    var channelType: EkoChannelType = .standard
    
    private var dataSource: ChannelListDataSource?
    private lazy var cellConfigurator = ChannelListTableViewCellConfigurator()

    // MARK: View Lifecycle

    override func viewDidLoad() {
        super.viewDidLoad()
        let nib = UINib(nibName: "SampleAppTableViewCell", bundle: nil)
        tableView.register(nib, forCellReuseIdentifier: cellIdentifier)
    }

    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        newQuery()
    }

    /// Fires a new request to fetch all the channels matching the current filters and tags preferences.
    func newQuery() {
        guard let dataSource = dataSource else { return }
        let channelFilter = UserDefaults.standard.filter
        let includingTags = UserDefaults.standard.includingTags
        let excludingTags = UserDefaults.standard.excludingTags

        if (channelFilter == .userIsNotMember || channelFilter == .all) && channelType == .private {
            dataSource.newQuery(channelType: .byTypes, channelFilter: channelFilter, includingTags: includingTags, excludingTags: excludingTags, channelTypeName: Set(arrayLiteral: "private"))
        } else {
            dataSource.newQuery(channelType: channelType, channelFilter: channelFilter, includingTags: includingTags, excludingTags: excludingTags)
        }
        
    }

    // MARK: UITableViewDataSource

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return dataSource?.numberOfChannels() ?? 0
    }

    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        guard let channel: EkoChannel = dataSource?.channel(for: indexPath) else { return UITableViewCell() }
        let cell: UITableViewCell = tableView.dequeueReusableCell(withIdentifier: cellIdentifier, for: indexPath)
        guard let sampleAppCell = cell as? SampleAppTableViewCell else { return UITableViewCell() }

        cellConfigurator.configure(sampleAppCell, with: channel)
        return sampleAppCell
    }

    // MARK: UITableViewDelegate

    override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        guard let channel: EkoChannel = dataSource?.channel(for: indexPath) else { return }
        join(channel: channel)
    }

    /// Joins the given channel.
    private func join(channel: EkoChannel) {
        let isComments: Bool = self.isComments(channel: channel)
        delegate?.joinChannel(channel.channelId, type: .standard, isComments: isComments)
    }

    /// Detects wheter the channel is a comments channel or not.
    private func isComments(channel: EkoChannel) -> Bool {
        if
            let tags = channel.tags as? [String],
            tags.contains("comments") {
            return true
        } else {
            return false
        }
    }

    // MARK: UIScrollViewDelegate

    /// Asks for more channels when we reach the bottom of the screen.
    override func scrollViewWillEndDragging(_ scrollView: UIScrollView,
                                            withVelocity velocity: CGPoint,
                                            targetContentOffset: UnsafeMutablePointer<CGPoint>) {
        let height = scrollView.frame.size.height
        let contentYoffset = scrollView.contentOffset.y
        let distanceFromBottom = scrollView.contentSize.height - contentYoffset
        if distanceFromBottom < height {
            dataSource?.loadMore()
        }
    }

    // MARK: DataSourceListener

    func dataSourceUpdated() {
        tableView.reloadData()
    }
}
