//
//  CreateChannelTableViewController.swift
//  SampleApp
//
//  Created by Federico Zanetello on 6/19/19.
//  Copyright © 2019 David Zhang. All rights reserved.
//

import UIKit
import EkoChat

private let checkmarkCellIdentifier: String = "checkCell"
private let textInputCellIdentifier: String = "textInputCell"
private let buttonCellIdentifier: String = "buttonCell"

final class CreateChannelTableViewController: UITableViewController,
                                              UITextFieldDelegate {
    weak var channelCreator: ChannelCreator?

    private var channelId: String = ""
    var channelIdTextFieldObserver: NSKeyValueObservation?

    private var userIds: [String] = []
    var userIdTextFieldObserver: [NSKeyValueObservation?] = []

    private let channelTypes: [EkoChannelCreateType] = [.standard, .private]
    private var selectedChannelType: EkoChannelCreateType? = .standard
    
    private var isConversation: Bool {
        return selectedChannelType == nil
    }
    
    private enum Section: CaseIterable {
        case channelId
        case type
        case userIds
        case addUserId
    }

    // MARK: UITableViewDataSource

    override func numberOfSections(in tableView: UITableView) -> Int {
        return Section.allCases.count
    }

    override func tableView(_ tableView: UITableView,
                            numberOfRowsInSection section: Int) -> Int {
        switch Section.allCases[section] {
        case .channelId:
            return 1
        case .type:
            return channelTypes.count + 1
        case .userIds:
            return userIds.count
        case .addUserId:
            return 1
        }
    }

    override func tableView(_ tableView: UITableView, titleForHeaderInSection section: Int) -> String? {
        switch Section.allCases[section] {
        case .channelId:
            return "ChannelId"
        case .type:
            return "Channel Type"
        case .userIds:
            return "UserIds"
        case .addUserId:
            return nil
        }
    }

    override func tableView(_ tableView: UITableView, titleForFooterInSection section: Int) -> String? {
        let section: Section = Section.allCases[section]
        switch section {
        case .userIds: return "Swipe left to remove a userId."
        case .addUserId,
             .channelId,
             .type: return nil
        }
    }

    override func tableView(_ tableView: UITableView,
                            cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        switch Section.allCases[indexPath.section] {
        case .channelId:
            return self.tableView(tableView,
                                  channelIdcellForRowAt: indexPath)
        case .type:
            return self.tableView(tableView,
                                  typeCellForRowAt: indexPath)
        case .userIds:
            return self.tableView(tableView,
                                  userIdCellForRowAt: indexPath)
        case .addUserId:
            return self.tableView(tableView,
                                  addUserIdCellForRowAt: indexPath)
        }
    }

    private func tableView(_ tableView: UITableView,
                           channelIdcellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell: UITableViewCell = tableView.dequeueReusableCell(withIdentifier: textInputCellIdentifier,
                                                                  for: indexPath)
        guard let textInputCell = cell as? TextInputTableViewCell else {
            return cell
        }

        let textField: UITextField = textInputCell.textField
        textField.delegate = self

        textInputCell.textField.placeholder = "channelId"
        textInputCell.textField.text = channelId
        channelIdTextFieldObserver = textField.observe(\.text,
                                                       options: [.new, .old]) { [weak self] textField, _ in
                                                        let newChannelId: String = textField.text ?? ""
                                                        self?.channelId = newChannelId
        }

        return textInputCell
    }

    private func tableView(_ tableView: UITableView,
                           typeCellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell: UITableViewCell = tableView.dequeueReusableCell(withIdentifier: checkmarkCellIdentifier,
                                                                  for: indexPath)

        let channelType: EkoChannelCreateType?
        if indexPath.row == channelTypes.count {
            channelType = nil
        } else {
            channelType = channelTypes[indexPath.row]
        }

        if channelType == nil {
            cell.textLabel?.text = "Conversation"
        } else {
            cell.textLabel?.text = String(describing: channelType!)
        }

        let accessoryType: UITableViewCell.AccessoryType
        if channelType == selectedChannelType {
            accessoryType = .checkmark
        } else {
            accessoryType = .none
        }

        cell.accessoryType = accessoryType

        return cell
    }

    private func tableView(_ tableView: UITableView,
                           userIdCellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell: UITableViewCell = tableView.dequeueReusableCell(withIdentifier: textInputCellIdentifier,
                                                                  for: indexPath)
        guard let textInputCell = cell as? TextInputTableViewCell else {
            return cell
        }

        let textField: UITextField = textInputCell.textField
        textField.delegate = self

        textField.placeholder = "userId"
        let userId = userIds[indexPath.row]
        textField.text = userId
        userIdTextFieldObserver[indexPath.row] = textField.observe(\.text,
                                                                   options: [.new, .old]) { [weak self] textField, _ in
                                                                    let newUserlId: String = textField.text ?? ""
                                                                    self?.userIds[indexPath.row] = newUserlId
        }

        return textInputCell
    }

    private func tableView(_ tableView: UITableView,
                           addUserIdCellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell: UITableViewCell = tableView.dequeueReusableCell(withIdentifier: buttonCellIdentifier,
                                                                  for: indexPath)
        cell.textLabel?.text = "Add userId"
        return cell
    }

    // MARK: UITableViewDelegate

    override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        defer { tableView.deselectRow(at: indexPath, animated: true) }

        let section: Section = Section.allCases[indexPath.section]

        switch section {
        case .addUserId:
            if isConversation && userIds.count >= 1 {
                return
            }
            userIds.append("")
            userIdTextFieldObserver.append(nil)
            guard let userIdsIndex = Section.allCases.firstIndex(of: .userIds) else { return }
            tableView.reloadSections([userIdsIndex],
                                     with: .automatic)
        case .type:
            if indexPath.row == channelTypes.count {
                selectedChannelType = nil
            } else {
                let newChannelType: EkoChannelCreateType = channelTypes[indexPath.row]
                selectedChannelType = newChannelType
            }
            tableView.reloadSections([indexPath.section],
                                     with: .automatic)
        case .channelId,
             .userIds:
            break // nothing to do
        }
    }

    override func tableView(_ tableView: UITableView, canEditRowAt indexPath: IndexPath) -> Bool {
        let section: Section = Section.allCases[indexPath.section]
        return section == .userIds
    }

    override func tableView(_ tableView: UITableView,
                            commit editingStyle: UITableViewCell.EditingStyle,
                            forRowAt indexPath: IndexPath) {
        guard editingStyle == .delete else { return }

        userIds.remove(at: indexPath.row)
        userIdTextFieldObserver.remove(at: indexPath.row)
        guard let userIdsIndex = Section.allCases.firstIndex(of: .userIds) else { return }
        tableView.reloadSections([userIdsIndex],
                                 with: .automatic)
    }

    @IBAction private func createTap(_ sender: UIBarButtonItem) {
        // Make sure that the keyboard is dismissed:
        // this is needed to register all the textField(s) values in the view.
        view.endEditing(true)
        if let selectedChannel = selectedChannelType {
            channelCreator?.createChannel(channelId: channelId, type: selectedChannel, userIds: userIds)
        } else if isConversation {
            guard let userId = userIds.first else { return }
            channelCreator?.createConversation(userId: userId)
        }
        navigationController?.popViewController(animated: true)
    }

    // MARK: UITextFieldDelegate

    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        textField.resignFirstResponder()
        return true
    }
}
