//
//  HomeViewController.swift
//  SampleApp
//
//  Created by Federico Zanetello on 4/30/19.
//  Copyright © 2019 David Zhang. All rights reserved.
//

import UIKit
import EkoChat
import UserNotifications
import os.log

final class HomeViewController: UIViewController,
                                ChannelsTableViewControllerDelegate,
                                MoreTableViewControllerDelegate,
                                EkoClientErrorDelegate,
                                ChannelCreator {
    
    @IBOutlet private weak var displayNameButton: UIButton!
    @IBOutlet private weak var totalMessageCountLabel: UILabel!

    private var connectionStatusObservationToken: NSKeyValueObservation?
    private var totalUnreadCountObservationToken: NSKeyValueObservation?

    // Only place where client should be held strongly
    var client: EkoClient!
    lazy var channelRepository = EkoChannelRepository(client: client)
    var channelType: EkoChannelType = .standard
    
    private var channelListViewController: ChannelListTableViewController!
    private var displayNameController: DisplayNameButtonController!
    private var unreadCountController: UnreadCountLabelController!

    // MARK: Lifecyle

    override func awakeFromNib() {
        super.awakeFromNib()
        registerForPushNotifications()
    }

    func registerForPushNotifications() {
        UNUserNotificationCenter.current()
            .requestAuthorization(options: [.alert, .sound, .badge]) { granted, _ in
                guard granted else { return }
                DispatchQueue.main.async {
                    UIApplication.shared.registerForRemoteNotifications()
                }
        }
    }

    override func viewDidLoad() {
        super.viewDidLoad()
        
        client.clientErrorDelegate = self
        
        trackConnectionStatus()
        trackUnreadCount()
        if let userId = UserDefaults.standard.userId {
            login(with: userId, displayName: nil)
        } else {
            login(with: "victimIOS", displayName: "iOS User 1")
        }

        displayNameController = DisplayNameButtonController(displayNameButton: displayNameButton,
                                                            userObject: client.currentUser!)
        unreadCountController = UnreadCountLabelController(unreadCountLabel: totalMessageCountLabel,
                                                           repository: channelRepository)
    }

    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        switch segue.destination {
        case let destination as ChannelListTableViewController:
            channelListViewController = destination
            channelListViewController.delegate = self
            channelListViewController.repository = channelRepository
            channelListViewController.channelType = channelType
        case let destination as ConnectionStatusLEDViewController:
            destination.client = client
            channelListViewController.repository = channelRepository
        case let destination as MoreTableViewController:
            destination.client = client
            destination.delegate = self
        case let destination as CreateChannelTableViewController:
            destination.channelCreator = self
        default:
            break
        }
    }

    private func trackUnreadCount() {
       totalUnreadCountObservationToken = channelRepository.observe(\.totalUnreadCount) { [weak self] channelRepository, _ in
            self?.totalMessageCountLabel.text = channelRepository.totalUnreadCount.description
        }
    }

    private func trackConnectionStatus() {
        connectionStatusObservationToken = client.observe(\.connectionStatus) { [weak self] client, _ in
            DispatchQueue.main.asyncAfter(deadline: .now() + .milliseconds(100)) {
                if let currentUser = client.currentUser {
                    self?.displayNameController.observe(userObject: currentUser)
                }
                self?.fireNewQuery(connectionStatus: client.connectionStatus)
            }
        }
    }

    private func fireNewQuery(connectionStatus: EkoConnectionStatus) {
        guard connectionStatus == .connected else { return }
        channelListViewController.newQuery()
    }

    @IBAction private func titleTapped() {
        let titleText = "Enter Your New Display Name"
        let alertController = UIAlertController(title: titleText, message: nil, preferredStyle: .alert)

        let renameAction = UIAlertAction(title: "Rename", style: .default) { [weak self] _ in
            guard let newDisplayName: String = alertController.textFields?.first?.text else { return }
            self?.client.setDisplayName(newDisplayName, completion: nil)
        }
        alertController.addAction(renameAction)

        alertController.addTextField { (textField) in
            textField.placeholder = self.client.currentUser?.object?.displayName
        }

        let cancelAction = UIAlertAction(title: "Cancel", style: .cancel, handler: nil)
        alertController.addAction(cancelAction)

        present(alertController, animated: true, completion: nil)
    }

    @IBAction private func profileTap() {
        let alertController = UIAlertController(title: "New login", message: nil, preferredStyle: .alert)

        alertController.addTextField(configurationHandler: { textField in
            textField.placeholder = "UserId"
        })

        let loginAction = UIAlertAction(title: "Login", style: .default, handler: { [weak self] _ in
            guard let userlId = alertController.textFields?.first?.text else { return }
            self?.client.unregisterDevice()
            self?.login(with: userlId, displayName: nil)
        })
        alertController.addAction(loginAction)

        let cancelAction = UIAlertAction(title: "Cancel", style: .cancel, handler: nil)
        alertController.addAction(cancelAction)

        present(alertController, animated: true, completion: nil)
    }
    
    // MARK: MoreTableViewControllerDelegate

    func moreTable(_ viewController: MoreTableViewController, willChangeChannelType channelType: EkoChannelType) {
        channelListViewController.channelType = channelType
        fireNewQuery(connectionStatus: client.connectionStatus)
    }
    
    // MARK: ChannelsTableViewControllerDelegate

    func joinChannel(_ channelId: String, type: EkoChannelType, isComments: Bool) {
        if isComments {
            joinComments(channelId: channelId, type: type)
        } else {
            joinChat(channelId: channelId, type: type)
        }
    }

    private func joinChat(channelId: String, type: EkoChannelType) {
        joinChat(storyboardIdentifier: "EkoChat",
                 channelId: channelId,
                 type: type)
    }

    private func joinComments(channelId: String, type: EkoChannelType) {
        joinChat(storyboardIdentifier: "EkoComments",
                 channelId: channelId,
                 type: type)
    }

    private func joinChat(storyboardIdentifier: String, channelId: String, type: EkoChannelType) {
        let storyboard = UIStoryboard(name: "Chats", bundle: nil)
        let viewController = storyboard.instantiateViewController(withIdentifier: storyboardIdentifier)
        guard let chatViewController = viewController as? EkoChatViewController else { return }
        chatViewController.channelId = channelId
        chatViewController.client = client
        chatViewController.joinChannel(channelId: channelId,
                                       type: type)
        navigationController?.pushViewController(chatViewController, animated: true)
    }

    private func login(with userId: String, displayName: String?) {
        client.registerDevice(withUserId: userId, displayName: displayName)
        UserDefaults.standard.set(userId, forKey: "userId")
    }

    // MARK: ChannelCreator

    func createChannel(channelId: String,
                       type: EkoChannelCreateType,
                       userIds: [String]) {        
        let channelType = EkoChannel.channelType(for: type)
        let channelObject: EkoObject<EkoChannel> = channelRepository.createChannel(channelId,
                                                                                   type: type,
                                                                                   users: userIds)
        var token: EkoNotificationToken?
        token = channelObject.observe { [weak self] channelObject, _ in
            token?.invalidate()
            switch channelObject.dataStatus {
            case .local,
                 .fresh:
                self?.joinChannel(channelId: channelId,
                                  channelObject: channelObject,
                                  type: channelType)
            case .error,
                 .notExist:
                fallthrough
            @unknown default:
                self?.displayOkAlert(title: "Cannot create channel with id '\(channelId)'",
                                     message: "A channel with the same id may already exist")
            }
        }
    }
    
    func createConversation(userId: String) {
        let channelObject: EkoObject<EkoChannel> = channelRepository.createConversation(userId, displayName: nil, metadata: [:], tags: [])
        var token: EkoNotificationToken?
        token = channelObject.observe { [weak self] channelObject, _ in
            token?.invalidate()
            switch channelObject.dataStatus {
            case .local,
                 .fresh:
                guard let channelId = channelObject.object?.channelId else { return }
                self?.joinChannel(channelId: channelId,
                                  channelObject: channelObject,
                                  type: .conversation)
            case .error,
                 .notExist:
                fallthrough
            @unknown default:
                self?.displayOkAlert(title: "Cannot create conversation",
                                     message: "A channel with the same id may already exist")
            }
        }
    }

    private func joinChannel(channelId: String,
                             channelObject: EkoObject<EkoChannel>,
                             type: EkoChannelType) {
        let isCommentsChannel: Bool
        if
            let channel: EkoChannel = channelObject.object,
            let tags: [String] = channel.tags as? [String],
            tags.contains("comments") {
            isCommentsChannel = true
        } else {
            isCommentsChannel = false
        }

        joinChannel(channelId, type: type, isComments: isCommentsChannel)
    }

    // MARK: EkoClientErrorDelegate

    func didReceiveAsyncError(_ error: Error) {
        let error = error as NSError
        guard
            let ekoError = EkoErrorCode(rawValue: error.code)
            else {
                assertionFailure("unknown error \(error.code), please report this code to Eko")
                return
        }

        os_log("%@", type: .error, String(describing: ekoError))
        displayAlert(for: ekoError)
    }

    private func displayAlert(for error: EkoErrorCode) {
        displayOkAlert(title: "Eko Error", message: String(describing: error))
    }

    private func displayOkAlert(title: String, message: String) {
        let alert = UIAlertController(title: title, message: message, preferredStyle: .alert)
        let action = UIAlertAction(title: "Ok", style: .default, handler: nil)
        alert.addAction(action)
        present(alert, animated: true)
    }
}
