//
//  UserPostsFeedDataSource.swift
//  SampleApp
//
//  Created by Nishan Niraula on 5/5/20.
//  Copyright © 2020 David Zhang. All rights reserved.
//

import Foundation

class UserPostsFeedManager {
    
    let client: EkoClient
    let feedRepository: EkoFeedRepository
    let postCreator: EkoPostCreator
    
    var postCollection: EkoCollection<EkoPost>?
    var feedCollectionToken: EkoNotificationToken?
    
    var editedPost: EkoPost?
    let userId: String?
    let userName: String?
    
    var reactor: EkoPostReactor?
    
    var reactionCollection: EkoCollection<EkoPostReaction>?
    var reactionCollectionToken: EkoNotificationToken?
    
    lazy var dateFormatter: DateFormatter = {
        let formatter = DateFormatter()
        formatter.dateFormat = "MMM d, HH:mm"
        return formatter
    }()
    
    // Make sure your EkoClient is setup and then setup EkoFeedRepository & EkoPostCreator
    init(client: EkoClient, userId: String?, userName: String? = nil) {
        self.client = client
        self.userId = userId
        self.userName = userName
        self.feedRepository = EkoFeedRepository(client: client)
        self.postCreator = feedRepository.postCreatorManager()
    }
    
    // MARK:- Feed Observer
    
    // Listen to feed changes. Anytime a post is created, observer
    // would be notified.
    // Note: Retain that EkoNotificationToken
    func observePostsFeedChanges(changeHandler:@escaping ()->()) {
        sortCurrentFeed(option: .lastCreated)
        feedCollectionToken = postCollection?.observe({(collection, _, error) in
            print("[POST] Change Observed in Posts")
            changeHandler()
        })
    }
    
    func getPostAtIndex(index: Int) -> EkoPost? {
        return postCollection?.object(at: UInt(index))
    }
    
    // MARK:- Reaction Observers
    
    func observeReactionsForPost(post: EkoPost?, reaction: String, completion:@escaping ()->()) {
        guard let post = post else { return }
        
        reactionCollectionToken?.invalidate()
        
        reactionCollection = feedRepository.getReactionsForPost(withPostId: post.postId, reactionName: reaction)
        reactionCollectionToken = reactionCollection?.observe({ (collection, change, error) in
            completion()
        })
    }
    
    func observeReactionsForFeedAtIndex(index: Int) {
        guard let post = postCollection?.object(at: UInt(index)) else { return }
        
        print("[Reaction] Fetching all reactions for this post")
        reactionCollection = feedRepository.getAllReactionsForPost(withPostId: post.postId)
        reactionCollectionToken = reactionCollection?.observe({ (collection, _, error) in
            print("[POST] Changes Observed in Reactions For Post")
        })
    }
    
    func getReactionAtIndex(index: Int) -> EkoPostReaction? {
        return reactionCollection?.object(at: UInt(index))
    }
    
    // Creates the post
    func createPost(text: String, onCompletion:@escaping (_ isSuccess: Bool)->()) {
        let postBuilder: EkoPostCreationBuilder
        if let id = userId {
            postBuilder = EkoPostUserFeedBuilder(text: text, userId: id)
        } else {
            postBuilder = EkoPostPersonalFeedBuilder(text: text)
        }
        
        postCreator.createPost(with: postBuilder) { (isSuccess, error) in
            onCompletion(isSuccess)
        }
    }
    
    func updatePost(text: String, onCompletion: @escaping (_ isSuccess: Bool)->()) {
        guard let post = editedPost else { return }
        
        postCreator.updatePost(withPostId: post.postId, text: text) { (isSuccess, error) in
            onCompletion(isSuccess)
        }
        
        editedPost = nil
    }
    
    func deletePost(at index: Int, onCompletion: @escaping (_ isSuccess: Bool)->()) {
        guard let post = postCollection?.object(at: UInt(index)) else { return }
        
        postCreator.deletePost(withPostId: post.postId) { (isSuccess, error) in
            onCompletion(isSuccess)
        }
    }
    
    func addReactionToPost(at index: Int, reaction: String) {
        guard let post = postCollection?.object(at: UInt(index)) else { return }
        
        // If you have already added the same reaction to the post, remove it
        if post.myReactions.contains(reaction) {
            removeReactionToPost(at: index, reaction: reaction)
        } else {
            reactor = EkoPostReactor(client: client, post: post)
            reactor?.addReaction(reaction, completion: { (isSuccess, error) in
                print("[Reaction] Completion Called \(isSuccess)")
            })
        }
    }
    
    func removeReactionToPost(at index: Int, reaction: String) {
        guard let post = postCollection?.object(at: UInt(index)) else { return }
        
        reactor = EkoPostReactor(client: client, post: post)
        reactor?.removeReaction(reaction, completion: { (isSuccess, error) in
            print("[Reaction] Completion Called \(isSuccess)")
        })
    }
    
    func sortCurrentFeed(option: EkoUserFeedSortOption) {
        if let id = userId {
            postCollection = feedRepository.getUserFeed(id, sortBy: option)
        } else {
            postCollection = feedRepository.getMyFeedSorted(by: option)
        }
    }
    
    func loadMorePosts() {
        guard let hasMorePosts = postCollection?.hasNext, hasMorePosts else { return }
        
        postCollection?.nextPage()
    }
    
    func getNumberOfFeedItems() -> Int {
        let count = Int(postCollection?.count() ?? 0)
        return count
    }
    
    func getFeedItemHeaderData(at index: Int) -> (title: String, date: String) {
        let post = postCollection?.object(at: UInt(index))
        
        let userName = post?.postedUser?.displayName ?? "No Name"
        
        var dateStr = "-"
        if let date = post?.createdAt {
            dateStr = dateFormatter.string(from: date)
        }
        
        return (userName, dateStr)
    }
    
    func getFeedItemData(at index: Int) -> FeedItemModel {
        let post = postCollection?.object(at: UInt(index))
        let postModel = createTextFeedModel(post: post)
        
        return postModel
    }
    
    func getReactionData(at index: Int) -> String {
        guard let post = postCollection?.object(at: UInt(index)), let reactions = post.reactions as? [String: Int] else { return "" }
        let data = reactions.map { $1 > 0 ? "\($0): \($1)" : "" }.joined(separator: " ")
        return data
    }
    
    func getFeedItemViewModels(at index: Int) -> [FeedCellItem] {
        guard let post = postCollection?.object(at: UInt(index)) else { return [] }
        
        var cellItems: [FeedCellItem] = [.header, .content(type: .text)]
        
        if post.reactionsCount > 0 {
            cellItems.append(.reaction)
        }
        
        cellItems.append(.footer)
        return cellItems
    }
    
    func prepareToEditPost(at index: Int) {
        let post = postCollection?.object(at: UInt(index))
        self.editedPost = post
    }
    
    func getEditPostData() -> TextFeedModel {
        let postModel = createTextFeedModel(post: editedPost)
        return postModel
    }
    
    func getFeedTitle() -> String {
        if let _ = userId {
            let name = userName ?? "-"
            return ("\(name)'s Feed")
        } else {
            return ("My Feed")
        }
    }
    
    // MARK:- Private Helpers
    
    private func createTextFeedModel(post: EkoPost?) -> TextFeedModel {
        let textData = post?.data?["text"] as? String ?? "-"
        let userName = post?.postedUser?.displayName ?? "No Name"
        
        var dateStr = "-"
        if let date = post?.createdAt {
            dateStr = dateFormatter.string(from: date)
        }
        
        let postModel = TextFeedModel(text: textData, userName: userName, date: dateStr)
        return postModel
    }
    
    func getReadableDate(date: Date?) -> String {
        if let date = date {
            return dateFormatter.string(from: date)
        } else {
            return ""
        }
    }
}
