//
//  UserPostsFeedViewController.swift
//  SampleApp
//
//  Created by Nishan Niraula on 4/27/20.
//  Copyright © 2020 David Zhang. All rights reserved.
//

import UIKit

// NOTE
//
// This class sets up the UI part for the Current User Feed. It shows the posts for
// logged in user. Look into `UserPostsFeedManager` for actual interaction with SDK.
class UserPostsFeedViewController: UIViewController {
    
    @IBOutlet weak var tableView: UITableView!
    @IBOutlet weak var sortInfoLabel: UILabel!
    
    @IBOutlet weak var userInfoContainerView: UIView!
    @IBOutlet weak var userAvatarView: UIImageView!
    @IBOutlet weak var userNameLabel: UILabel!
    @IBOutlet weak var userIdLabel: UILabel!
    
    var client: EkoClient!
    var feedManager: UserPostsFeedManager!
    var userName: String?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        setupViews()

    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        
        feedManager.observePostsFeedChanges { [weak self] in
            self?.updateUserFeed()
        }
    }
    
    func updateUserFeed() {
        self.tableView.reloadData()
    }
    
    func setupViews() {
        if let id = feedManager.userId {

            userAvatarView.layer.cornerRadius = 25
            userAvatarView.layer.masksToBounds = true
            userAvatarView.image = UIImage(named: "feed_profile")?.withRenderingMode(.alwaysOriginal)
            
            let feedName = feedManager.getFeedTitle()
            userNameLabel.text = feedName
            userIdLabel.text = id
        } else {
            self.title = "My Feed"
            userInfoContainerView.removeFromSuperview()
        }
        
        let sortButton = UIBarButtonItem(image: UIImage(named: "feed_sort"), style: .plain, target: self, action: #selector(onSortButtonTap))
        let addButton = UIBarButtonItem(barButtonSystemItem: .add, target: self, action: #selector(onAddButtonTap))
        self.navigationItem.rightBarButtonItems = [addButton, sortButton]
        
        tableView.dataSource = self
        tableView.delegate = self
        tableView.separatorStyle = .none
        tableView.reloadData()
        
        tableView.register(UITableViewCell.self, forCellReuseIdentifier: "ReactionCell")
        
        sortInfoLabel.font = UIFont.systemFont(ofSize: 15, weight: .medium)
        updateSortInfoLabel(option: "Last Created")
    }
    
    func updateSortInfoLabel(option: String) {
        sortInfoLabel.text = "Sorted By: \(option)"
    }
    
    @objc func onAddButtonTap() {
        displayNewPostsScreen(isEditMode: false)
    }
    
    @objc func onSortButtonTap() {
        let actions: [FeedItemDefaultAction] = [.sortFirstCreated, .sortLastCreated]
        displayMoreActions(title: "Sort the feed by", actions: actions, at: -1)
    }
    
    func displayNewPostsScreen(isEditMode: Bool) {
        let controller = self.storyboard?.instantiateViewController(withIdentifier: NewUserPostsViewController.identifier) as! NewUserPostsViewController
        controller.currentMode = isEditMode ? .edit : .create
        controller.feedManager = feedManager
        if #available(iOS 13.0, *) {
            // Prevents screen to dismiss when swiping down on ios 13
            controller.isModalInPresentation = true
        }
        
        self.present(UINavigationController(rootViewController: controller), animated: true, completion: nil)
    }
    
    func showAlert(alertMessage: String) {
        
        let alert = UIAlertController(title: "", message: alertMessage, preferredStyle: .alert)
        let action = UIAlertAction(title: "Dismiss", style: .cancel, handler: nil)
        alert.addAction(action)
        
        self.present(alert, animated: true, completion: nil)
    }
    
    func scrollViewWillEndDragging(_ scrollView: UIScrollView, withVelocity velocity: CGPoint, targetContentOffset: UnsafeMutablePointer<CGPoint>) {
        
        let frameHeight = scrollView.frame.size.height
        let contentOffset = scrollView.contentOffset.y
        let contentHeight = scrollView.contentSize.height
        
        let distanceFromBottom = contentHeight - contentOffset
        if distanceFromBottom < frameHeight {
            feedManager.loadMorePosts()
        }
    }
    
}

extension UserPostsFeedViewController: UITableViewDataSource, UITableViewDelegate {
    
    func numberOfSections(in tableView: UITableView) -> Int {
        return feedManager.getNumberOfFeedItems()
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        let feedItemViewModel = feedManager.getFeedItemViewModels(at: section)
        return feedItemViewModel.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        
        let cellItems = feedManager.getFeedItemViewModels(at: indexPath.section)
        let currentCellItem = cellItems[indexPath.row]
        
        switch currentCellItem {
        case .header:
            let cell = tableView.dequeueReusableCell(withIdentifier: FeedPostHeaderCell.identifier) as! FeedPostHeaderCell
            
            let data = feedManager.getFeedItemHeaderData(at: indexPath.section)
            cell.configure(title: data.title, date: data.date)
            
            cell.moreButtonAction = { [weak self] in
                let actions = [FeedItemDefaultAction.edit, FeedItemDefaultAction.delete]
                self?.displayMoreActions(title: "What would you like to do?", actions: actions, at: indexPath.section)
            }
            return cell
            
        case .content:
            let cell = tableView.dequeueReusableCell(withIdentifier: FeedPostMultilineLabelCell.identifier) as! FeedPostMultilineLabelCell
            
            let data = feedManager.getFeedItemData(at: indexPath.section) as? TextFeedModel
            cell.feedTextLabel.text = data?.text
            
            return cell
            
        case .reaction:
            let cell = tableView.dequeueReusableCell(withIdentifier: "ReactionCell")!
            cell.backgroundColor = .groupTableViewBackground
            cell.textLabel?.font = UIFont.systemFont(ofSize: 14)
            
            let data = feedManager.getReactionData(at: indexPath.section)
            cell.textLabel?.text = data
            
            return cell
            
        case .footer:
            let cell = tableView.dequeueReusableCell(withIdentifier: FeedPostFooterCell.identifier) as! FeedPostFooterCell
            
            cell.likeButtonAction = { [weak self] in
                self?.executeFeedAction(action: FeedItemDefaultAction.like, at: indexPath.section)
            }
            
            cell.commentButtonAction = { [weak self] in
                self?.executeFeedAction(action: FeedItemDefaultAction.comment, at: indexPath.section)
            }
            
            return cell
        }
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        let cellItems = feedManager.getFeedItemViewModels(at: indexPath.section)
        let currentCellItem = cellItems[indexPath.row]
        
        switch currentCellItem {
        case .reaction:
            let controller = self.storyboard?.instantiateViewController(withIdentifier: UserPostReactionsViewController.identifier) as! UserPostReactionsViewController
            
            let post = feedManager.getPostAtIndex(index: indexPath.section)
            controller.feedManager = feedManager
            controller.post = post
            
            self.navigationController?.pushViewController(controller, animated: true)
            
        default:
            break
        }
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        let cellItems = feedManager.getFeedItemViewModels(at: indexPath.section)
        let currentCellItem = cellItems[indexPath.row]
        return currentCellItem.height
    }
}

extension UserPostsFeedViewController {
    
    func displayMoreActions(title: String, actions: [FeedItemAction], at index: Int) {
        let alertController = UIAlertController(title: "", message: title, preferredStyle: .actionSheet)
        
        for action in actions {
            let sheetAction = UIAlertAction(title: action.title, style: .default) { [weak self] _ in
                self?.executeFeedAction(action: action, at: index)
            }
            alertController.addAction(sheetAction)
        }
        
        let cancelAction = UIAlertAction(title: "Cancel", style: .cancel, handler: nil)
        alertController.addAction(cancelAction)
        
        self.present(alertController, animated: true, completion: nil)
    }
    
    func executeFeedAction(action: FeedItemAction, at index: Int) {
        
        switch action.id {
        case FeedItemDefaultAction.edit.id:
            
            feedManager.prepareToEditPost(at: index)
            displayNewPostsScreen(isEditMode: true)
            
        case FeedItemDefaultAction.delete.id:
            
            feedManager.deletePost(at: index) { [weak self] (isSuccess) in
                let alertMessage = isSuccess ? "Post successfully deleted" : "Error while deleting post"
                self?.showAlert(alertMessage: alertMessage)
            }
            
        case FeedItemDefaultAction.like.id:
            feedManager.addReactionToPost(at: index, reaction: "like")
            
        case FeedItemDefaultAction.comment.id:
            feedManager.addReactionToPost(at: index, reaction: "love")
            
        case FeedItemDefaultAction.sortFirstCreated.id:
            feedManager.sortCurrentFeed(option: .firstCreated)
            updateSortInfoLabel(option: "First Created")
            
        case FeedItemDefaultAction.sortLastCreated.id:
            feedManager.sortCurrentFeed(option: .lastCreated)
            updateSortInfoLabel(option: "Last Created")
            
        default:
            fatalError("Implementation not found for action \(action.title)")
        }
    }
}

