//
//  UserListManager.swift
//  SampleApp
//
//  Created by Nishan Niraula on 5/11/20.
//  Copyright © 2020 David Zhang. All rights reserved.
//

import Foundation

class UserListManager {
    
    let client: EkoClient
    let userRepository: EkoUserRepository
    
    var userCollection: EkoCollection<EkoUser>?
    var userCollectionToken: EkoNotificationToken?
    
    var searchCollection: EkoCollection<EkoUser>?
    var searchCollectionToken: EkoNotificationToken?
    
    var isInSearchMode = false
    var searchText = ""
    var sortOption: EkoUserSortOption = .displayName
    
    let debouncer = Debouncer(delay: 0.3)
    
    init(client: EkoClient) {
        self.client = client
        self.userRepository = EkoUserRepository(client: client)
    }
    
    func fetchUserList(sortedBy option: EkoUserSortOption, completion:@escaping ()->()) {
        userCollection = userRepository.getAllUsersSorted(by: option)
        userCollectionToken = userCollection?.observe({ (collection, change, error) in
            guard !self.isInSearchMode else { return }
            
            completion()
        })
    }
    
    func searchUserList(name: String, completion:@escaping ()->()) {
        self.searchText = name
        self.isInSearchMode = !name.isEmpty
        
        if isInSearchMode {
            debouncer.setCallback { [weak self] in
                self?.waitAndSearchUser(name: name, completion: completion)
            }
            debouncer.call()
        } else {
            completion()
        }
    }
    
    func waitAndSearchUser(name: String, completion:@escaping ()->()) {
        searchCollection = userRepository.searchUser(name, sortBy: self.sortOption)
        searchCollectionToken = searchCollection?.observe({ (collection, change, error) in
            completion()
        })
    }
    
    func numberOfUsers() -> Int {
        if isInSearchMode {
            return Int(searchCollection?.count() ?? 0)
        } else {
            return Int(userCollection?.count() ?? 0)
        }
    }
    
    func getUserItem(at index: Int) -> EkoUser? {
        if isInSearchMode {
            return searchCollection?.object(at: UInt(index))
        } else {
            return userCollection?.object(at: UInt(index))
        }
    }
    
    func loadMoreUsers() {
        if isInSearchMode {
            guard let hasMorePosts = searchCollection?.hasNext, hasMorePosts else { return }
            searchCollection?.nextPage()
        } else {
            guard let hasMorePosts = userCollection?.hasNext, hasMorePosts else { return }
            userCollection?.nextPage()
        }
        
    }
}


// Simple debouncer for search
@objc public class Debouncer: NSObject {
    
    @objc public var delay: Double
    
    private var callback: (() -> Void)?
    
    private weak var timer: Timer?
    
    @objc public init(delay: TimeInterval) {
        self.delay = delay
    }
    
    deinit {
        timer?.invalidate()
    }
    
    @objc public func setCallback(_ callback: (() -> Void)?) {
        self.callback = callback
    }
    
    @objc public func call() {
        timer?.invalidate()
        let nextTimer = Timer.scheduledTimer(timeInterval: delay, target: self, selector: #selector(fire), userInfo: nil, repeats: false)
        timer = nextTimer
    }
    
    @objc public func fire() {
        self.callback?()
    }
    
}
